--[[

PSP Sudoku
Lua version
by Dave Kalin (kalin@dkdigital.com)

Stripped and rebuilt on the foundation of
PSP Minesweeper v1.3 by JEK, warpony@gmail.com

]]

grid={}
grid_difficulty={}
grid_value={}
dofile("puzzles.lua")

---- Defines the various colors used in the game ----
-------------------------------------------------------------------------------

colors = {
	white = Color.new(255,255,255),
	verylightgray = Color.new(240,240,240),
	lightgray = Color.new(200,200,200),
	mediumgray = Color.new(140,140,140),
	lightgreen = Color.new(160,240,160),
	lightyellow = Color.new(230,230,140),
	lightred = Color.new(240,160,160),
	white200 = Color.new(255,255,255,200),
	white150 = Color.new(255,255,255,150),
	white100 = Color.new(255,255,255,100),
	white50 = Color.new(255,255,255,50),
	black200 = Color.new(0,0,0,200),
	black150 = Color.new(0,0,0,150),
	black100 = Color.new(0,0,0,100),
	black50 = Color.new(0,0,0,50),
	clear = Color.new(0, 0, 0, 48),
	bgtint = Color.new(0, 150, 150),
	badflash = Color.new(50,100,200,200),
	crystflash = Color.new(255,150,150),
	deadcryst = Color.new(200,200,200,100),
	wallhitflash = Color.new(255, 255, 255, 100),
	newcircleflash = Color.new(255,255,255,150),
	line = Color.new(111, 244, 166),
	activeline = Color.new(111, 166, 244),
	yellowish = Color.new(240, 255, 20),
	nuked = Color.new(180, 240, 20),
	nbflash = Color.new(180, 240, 20, 100),
	red = Color.new(255, 40, 40)
}





--------------------------------------------------
-- Load the Font blitting code originally from Shine (http://forums.ps2dev.org)

dofile("fonts.lua")
dofile("logic.lua")

--------------------------------------------------

TimerFont = nil
BoardFont_White = nil
BoardFont_Black = nil
BoardFont_Red = nil
BoardFont_Green = nil
collectgarbage()


-- Bitmap Fonts
TimerFont = BitFont:new("fonts/TimerFont.png")

BoardFont_White = BitFont:new("fonts/BoardFont_White.png", {hSize = 16, lSpace = 1, vPos = {0, 16, 31}, vSize = {16,16,16}})
BoardFont_Black = BitFont:new("fonts/BoardFont_Black.png", {hSize = 16, lSpace = 1, vPos = {0, 16, 31}, vSize = {16,16,16}})
BoardFont_Red   = BitFont:new("fonts/BoardFont_Red.png", {hSize = 16, lSpace = 1, vPos = {0, 16, 31}, vSize = {16,16,16}})
BoardFont_Green = BitFont:new("fonts/BoardFont_Green.png", {hSize = 16, lSpace = 1, vPos = {0, 16, 31}, vSize = {16,16,16}})
--

-- TrueType Fonts (unimplemented so far)
-- Truetype has been disabled for now, since the Bitmap fonts work so well
--proportional = Font.createProportional()
--proportional:setPixelSizes(0, 12)
--
--testfont = Font.load("fonts/testfont.ttf")
--testfont:setPixelSizes(0, 12)
--

System.usbDiskModeActivate()

-- Loading images
background = Image.load("images/background.png")
optionsDialog = Image.load("images/optionsDialog.png")
newDialog = Image.load("images/newDialog.png")
prefOn = Image.load("images/on.png")
prefOff = Image.load("images/off.png")


-- Constants
black = Color.new(0, 0, 0)
white = Color.new(255, 255, 255)
red = Color.new(255, 0, 0,170)
lightRed = Color.new(255, 128, 128, 64) -- half transparent
green = Color.new(0, 255, 0)
infoRect = {w=154, h=37, x=295, y= 213}
infoCenterX = infoRect.x + (infoRect.w / 2)
infoCenterY = infoRect.y + (infoRect.h / 2)  

-- Global Variables
puzzleTimer = Timer.new()
timerOn = true
showCandidates = true

offscr = Image.createEmpty(480,272) 
offscr:clear()

squaresize = 25
selX = 1
selY = 1

drawHelpSquares = false

boardoffsetx = 30
boardoffsety = 23


------


function createAgrid(difficulty)  -- creates a valid soduku grid
  solution = {}
  isaClue = {}
  math.randomseed(os.time())
	difficulty_string={}
	difficulty_string["Easy"]=1
	difficulty_string["Medium"]=2
	difficulty_string["Hard"]=3
	difficulty_string["Very Hard"]=4
	--repeat
		grid_num=math.random(table.getn(grid))
	--until grid_difficulty[grid_num]==difficulty_string[difficulty]

  for i=1,9 do
      solution[i] = {}
      isaClue[i] = {}
      line=grid[grid_num][i]
      for j=1,9 do
         solution[i][j] = string.byte(line, j) - string.byte("0")
        if solution[i][j]==0 then
			isaClue[i][j] = 0
		else
			isaClue[i][j] = 1
			guess[i][j]=solution[i][j]
		end
      end
  end
   
  -- test grid goes here
  -- I have a fixed grid right now, but I'm still looking for some code to generate puzzles.
  -- NOTE: Can this be done with tables instead?
  --[[solution [1] = {4,6,5,1,7,8,9,3,2}
  solution [2] = {2,1,3,9,6,4,5,8,7}
  solution [3] = {9,8,7,5,2,3,4,6,1}
  solution [4] = {8,4,2,7,3,9,6,1,5}
  solution [5] = {5,7,9,6,8,1,3,2,4}
  solution [6] = {1,3,6,4,5,2,8,7,9}
  solution [7] = {6,5,1,8,4,7,2,9,3}
  solution [8] = {7,2,8,3,9,5,1,4,6}
  solution [9] = {3,9,4,2,1,6,7,5,8}
  

  isaClue [1] = {1,1,0,1,1,0,0,0,0}
  isaClue [2] = {1,0,0,1,0,0,1,1,1}
  isaClue [3] = {1,1,1,0,0,0,1,0,0}
  isaClue [4] = {0,0,0,0,1,1,0,0,1}
  isaClue [5] = {0,0,0,0,1,0,0,0,0}
  isaClue [6] = {1,0,0,1,1,0,0,0,0}
  isaClue [7] = {0,0,1,0,0,0,1,1,1}
  isaClue [8] = {1,1,1,0,0,1,0,0,1}
  isaClue [9] = {0,0,0,0,1,1,0,1,1}]]
  -- test grid
  
  
end


-- Do this stuff at the start of every new puzzle 
function init()  
   --Initialize variables

   showCorrect = false
	foundSolution=false

   -- Create array
  guess = {}
  for i=1,9 do
      guess[i] = {}
      for j=1,9 do
         guess[i][j] = 0
      end
   end

   x= puzzleTimer:reset()
   x= puzzleTimer:start()

end


function doNewDialog()
	inDialog = true
	difficulty = 0
	x= puzzleTimer:stop()

	screen:blit(100, 60, newDialog, true)
    	screen.flip()

	while inDialog do
		
		pad = Controls.read()

		if pad ~= oldPad then
			oldPad = pad
			
			if pad:cross() then
				inDialog = false
			end
			if pad:down() then
				difficulty = 1
				inDialog = false
			end
			if pad:left() then
				difficulty = 2
				inDialog = false
			end
			if pad:right() then
				difficulty = 3
				inDialog = false
			end
			if pad:up() then
				difficulty = 4
				inDialog = false
			end
		end
		
		screen.waitVblankStart()
	
	end
	x= puzzleTimer:start()


	if difficulty > 0 then 
		init()

		createAgrid(difficulty)
	end
	
end



function doOptionsDialog()
	inDialog = true
	x= puzzleTimer:stop()
	screen:blit(100, 60, optionsDialog, true)

	while inDialog do
		offscr:clear()
		screen:blit(100, 60, optionsDialog, true)
		
		pad = Controls.read()

		if pad ~= oldPad then
			oldPad = pad
			
			
			if pad:circle() then
				if drawHelpSquares == true then 
					drawHelpSquares = false
				else
					drawHelpSquares = true
				end
			end
			
			if pad:square() then
				if timerOn == true then 
					timerOn = false
				else
					timerOn = true
				end
			end
			
			if pad:cross() then
				inDialog = false
			end

			if pad:triangle() then
				if showCandidates == true then 
					showCandidates = false
				else
					showCandidates = true
				end
			end
			
			if pad:cross() then
				inDialog = false
			end
		end
		

		-- draw all the preferences in the dialog
		
		if drawHelpSquares == true then 
			screen:blit(335, 124, prefOn, true)
		else
			screen:blit(335, 124, prefOff, true)
		end
		
		if timerOn == true then 
			screen:blit(335, 154, prefOn, true)
		else
			screen:blit(335, 154, prefOff, true)
		end
		
		if showCandidates == true then 
			screen:blit(355, 184, prefOn, true)
		else
			screen:blit(355, 184, prefOff, true)
		end
		
		
		screen.waitVblankStart()
		screen.flip()
		

	end
	x= puzzleTimer:start()

end



function doDrawTimer()
	
	    
	elapsedTime = puzzleTimer:time() / 1000
	
	mins = math.floor( elapsedTime / 60)
	secs = string.format( "%02d", math.mod(elapsedTime, 60) )
	theString = "Time " .. mins .. ":" .. secs

	
    	if showCandidates == true then 
    		infoY = infoCenterY - 12 -- scoot the timer up if the candidates are on
    	else
    		infoY = infoCenterY - 4 
    	end 

	TimerFont:print(theString, infoCenterX, infoY, BitFont.CENTER, white)

	---------------------
	-- Truetype has been disabled for now, since the Bitmap fonts work so well
	-- screen:fontPrint(proportional, 370,230, theString, white)
	---------------------

end

function doDrawInfoBox()
    if timerOn == true then 
    	doDrawTimer() 
    end

    if showCandidates == true then
	doDrawPossibleChoices()
    end
end

function doDrawHelpSquares()
	--selBox = int(selY / 3) + selX

	

	for r=1,9 do
		if  (r == selX) then 
			for c=1,9 do
			    left = ((r - 1) * squaresize) + boardoffsetx
			    top =  ((c - 1) * squaresize) + boardoffsety
			    offscr:fillRect(left, top, squaresize, squaresize, lightRed)
			end
		end
	end

	for c=1,9 do
		if  (c == selY) then 
			for r=1,9 do
			    left = ((r - 1) * squaresize) + boardoffsetx
			    top =  ((c - 1) * squaresize) + boardoffsety
			    offscr:fillRect(left, top, squaresize, squaresize, lightRed)
			end
		end
	end

	boxX = math.floor ( (selX-1) / 3) *3 + 1
	boxY = math.floor ( (selY-1) / 3) *3 + 1
	for c=boxY,boxY+2 do
			for r=boxX,boxX+2 do
			    left = ((r - 1) * squaresize) + boardoffsetx
			    top =  ((c - 1) * squaresize) + boardoffsety
			    offscr:fillRect(left, top, squaresize, squaresize, lightRed)
			end
	end
	
	
end



-- MAIN ROUTINE

-- Prepare game for starting
init()
createAgrid(1)


oldPad = Controls.read()   -- to prevent holding down the buttons

while true do
	
	-- Read controls
	pad = Controls.read()
	
	if pad ~= oldPad then
		oldPad = pad

		if pad:left() then
			selX = selX - 1
			if selX == 0 then
				selX = 9
			end
		elseif pad:right() then
			selX = selX + 1
			if selX > 9 then
				selX = 1
			end
		elseif pad:up() then
			selY = selY - 1
			if selY == 0 then
				selY = 9
			end
		elseif pad:down() then
			selY = selY + 1
			if selY > 9 then
				selY = 1
			end
		elseif pad:l() then
			guess[selX][selY] = guess[selX][selY] - 1
			if guess[selX][selY] < 0 then
			    guess[selX][selY] = 9
			end
		elseif pad:r() then
			guess[selX][selY] = guess[selX][selY] + 1
			if guess[selX][selY] > 9 then
			    guess[selX][selY] = 0
			end
			
		elseif pad:select() then
			doOptionsDialog()
			
		elseif pad:note() then
			doOptionsDialog()
			
		elseif pad:triangle() then
			doNewDialog()


		-- these check to see if the X button is held down (for "Show Correct" mode")
		elseif pad:cross() then
	                showCorrect = true  -- switch modes for showing right and wrong choices  
				if checkForSolution() then
					foundSolution=true
				else
					foundSolution=false
				end
		elseif showCorrect == true then
	                showCorrect = false  -- switch modes for showing right and wrong choices  
	                
	                
		end
		
	end

	
	--screen:clear()
	offscr:clear()

  
   -- Draw background image
   screen:blit(0, 0, background, 0, 0, background:width(), background:height(), false)

   -- Draw the elements in the Info Box (Timer, Candidates)
   doDrawInfoBox()


   -- Draw the red square around the currently selected square
    if drawHelpSquares == true then 
    	doDrawHelpSquares() 
    end

    left = ((selX - 1) * squaresize) + boardoffsetx
    top =  ((selY - 1) * squaresize) + boardoffsety
    
    offscr:fillRect(left, top, squaresize, squaresize, red)
    
    
   -- Draw the numbers
    for i=1,9 do
      for j=1,9 do
         left = ((i - 1) * squaresize) + boardoffsetx + 4
         top =  ((j - 1) * squaresize) + boardoffsety + 4
         
         if isaClue[i][j] == 1 then
				BoardFont_Black:print(solution[i][j], left,top)
                --blitString(offscr, left,top,solution[i][j], blackfont)  
         		-- offscr:print(left,top,solution[i][j],black )
         elseif guess[i][j] > 0 then
			
			
                if showCorrect == true then 
		     if foundSolution then
					BoardFont_Green:print(guess[i][j], left,top)
		     else
					BoardFont_Red:print(guess[i][j], left,top)
		     end
		else
				BoardFont_White:print(guess[i][j], left,top)
		end
	 end
      end
    end


   --Draw "buffer" to screen
   screen:blit(0, 0, offscr, 0, 0, offscr:width(), offscr:height(), true)
    screen.waitVblankStart()
    screen.flip()
 
    if pad:start() then break end  -- just to quit the program
   
    if pad:circle() then
   	offscr:save("offscr.png")
   	screen:save("screen.png")
    end   

   
end










